SWAGGER_DIFF_URL = "https://github.com/Sayi/swagger-diff/releases/download/v1.2.1/swagger-diff.jar"

pipeline {
    agent { node { label "module_build_4_jenkins_agent" } }

    options {
        buildDiscarder logRotator(daysToKeepStr: "5", numToKeepStr: "10")
    }

    stages {
        stage("Check Diff") {
            steps {
                script {
                    // 3rdパーティライブラリ取得
                    def result = sh script: "wget -q ${SWAGGER_DIFF_URL}", returnStatus: true
                    if (result != 0) error "ライブラリ(swagger-diff.jar)が取得できませんでした。\nURLの変更が必要かどうか確認してください。"
                    
                    // diffを比較するための準備
                    sh "mkdir -p ${WORKSPACE}/tmp/new ${WORKSPACE}/tmp/old"
                    sh "cp *swagger.json ${WORKSPACE}/tmp/new"

                    // 最新のタグ存在チェック
                    def existsTag = sh script :"git describe origin/production --tag", returnStatus: true
                    if (existsTag != 0) {
                        print("タグが存在しないためdiffを比較する必要はありません。")
                        return
                    }

                    // 最新のタグ名取得
                    def latestTag = sh(script :"git describe origin/production --tag", returnStdout: true).trim()
                    print("最新タグ名: ${latestTag}")

                    // タグからjson取得
                    sh "git checkout tags/${latestTag} *swagger.json"
                    sh "cp *swagger.json ${WORKSPACE}/tmp/old"

                    // jsonのみ最新に戻す
                    sh "git checkout HEAD -- *swagger.json"

                    // diff生成
                    sh """java -jar swagger-diff.jar \
                               -old ${WORKSPACE}/tmp/old/*swagger.json \
                               -new ${WORKSPACE}/tmp/new/*swagger.json \
                               -output-mode html > diff.html"""
                    sh "rm -fr ${WORKSPACE}/tmp"
                }
            }
        }

        stage("Archive Artifacts") {
            steps {
                archiveArtifacts artifacts: "**", excludes: "Jenkinsfile.groovy", fingerprint: true, onlyIfSuccessful: true
            }
        }

        stage("Clean") {
            steps {
                deleteDir()
            }
        }
    }
}
